@@ -28,21 +28,8 @@ local patch = require("luarocks.tools.patch")
 
 local dir_stack = {}
 
-math.randomseed(os.time())
-
 local dir_separator = "/"
 
---- Quote argument for shell processing.
--- Adds single quotes and escapes.
--- @param arg string: Unquoted argument.
--- @return string: Quoted argument.
-function fs_lua.Q(arg)
-   assert(type(arg) == "string")
-
-   -- FIXME Unix-specific
-   return "'" .. arg:gsub("'", "'\\''") .. "'"
-end
-
 --- Test is file/dir is writable.
 -- Warning: testing if a file/dir is writable does not guarantee
 -- that it will remain writable and therefore it is no replacement
@@ -67,23 +54,6 @@ function fs_lua.is_writable(file)
    return result
 end
 
---- Create a temporary directory.
--- @param name string: name pattern to use for avoiding conflicts
--- when creating temporary directory.
--- @return string or (nil, string): name of temporary directory or (nil, error message) on failure.
-function fs_lua.make_temp_dir(name)
-   assert(type(name) == "string")
-   name = dir.normalize(name)
-
-   local temp_dir = (os.getenv("TMP") or "/tmp") .. "/luarocks_" .. name:gsub(dir.separator, "_") .. "-" .. tostring(math.floor(math.random() * 10000))
-   local ok, err = fs.make_dir(temp_dir)
-   if ok then
-      return temp_dir
-   else
-      return nil, err
-   end
-end
-
 local function quote_args(command, ...)
    local out = { command }
    for _, arg in ipairs({...}) do
@@ -274,6 +244,10 @@ function fs_lua.make_dir(directory)
          if not ok then
             return false, err
          end
+         ok, err = fs.chmod(path, cfg.perm_exec)
+         if not ok then
+            return false, err
+         end
       elseif mode ~= "directory" then
          return false, path.." is not a directory"
       end
@@ -319,7 +293,7 @@ function fs_lua.copy(src, dest, perms)
    if destmode == "directory" then
       dest = dir.path(dest, dir.base_name(src))
    end
-   if not perms then perms = fs.get_permissions(src) end
+   if not perms then perms = fs.attributes(src, "permissions") end
    local src_h, err = io.open(src, "rb")
    if not src_h then return nil, err end
    local dest_h, err = io.open(dest, "w+b")
@@ -339,12 +313,14 @@ end
 -- Assumes paths are normalized.
 -- @param src string: Pathname of source
 -- @param dest string: Pathname of destination
+-- @param perms string or nil: Optional permissions.
+-- If not given, permissions of the source are copied over to the destination.
 -- @return boolean or (boolean, string): true on success, false on failure
-local function recursive_copy(src, dest)
+local function recursive_copy(src, dest, perms)
    local srcmode = lfs.attributes(src, "mode")
 
    if srcmode == "file" then
-      local ok = fs.copy(src, dest)
+      local ok = fs.copy(src, dest, perms)
       if not ok then return false end
    elseif srcmode == "directory" then
       local subdir = dir.path(dest, dir.base_name(src))
@@ -352,7 +328,7 @@ local function recursive_copy(src, dest)
       if not ok then return nil, err end
       for file in lfs.dir(src) do
          if file ~= "." and file ~= ".." then
-            local ok = recursive_copy(dir.path(src, file), subdir)
+            local ok = recursive_copy(dir.path(src, file), subdir, perms)
             if not ok then return false end
          end
       end
@@ -363,9 +339,10 @@ end
 --- Recursively copy the contents of a directory.
 -- @param src string: Pathname of source
 -- @param dest string: Pathname of destination
+-- @param perms string or nil: Optional permissions. 
 -- @return boolean or (boolean, string): true on success, false on failure,
 -- plus an error message.
-function fs_lua.copy_contents(src, dest)
+function fs_lua.copy_contents(src, dest, perms)
    assert(src and dest)
    src = dir.normalize(src)
    dest = dir.normalize(dest)
@@ -373,7 +350,7 @@ function fs_lua.copy_contents(src, dest)
 
    for file in lfs.dir(src) do
       if file ~= "." and file ~= ".." then
-         local ok = recursive_copy(dir.path(src, file), dest)
+         local ok = recursive_copy(dir.path(src, file), dest, perms)
          if not ok then
             return false, "Failed copying "..src.." to "..dest
          end
@@ -728,6 +705,14 @@ end
 
 if md5_ok then
 
+-- Support the interface of lmd5 by lhf in addition to md5 by Roberto
+-- and the keplerproject.
+if not md5.sumhexa and md5.digest then
+   md5.sumhexa = function(msg)
+      return md5.digest(msg)
+   end
+end
+
 --- Get the MD5 checksum for a file.
 -- @param file string: The file to be computed.
 -- @return string: The MD5 checksum or nil + error
@@ -773,8 +758,31 @@ function fs_lua.chmod(file, mode)
    return err == 0
 end
 
-function fs_lua.get_permissions(file)
-   return posix.stat(file, "mode")
+function fs_lua.attributes(file, attrtype)
+   if attrtype == "permissions" then
+      return posix.stat(file, "mode") or nil
+   elseif attrtype == "owner" then
+      local uid = posix.stat(file, "uid")
+      if not uid then return nil end
+      return posix.getpwuid(uid).pw_name or nil
+   else
+      return nil
+   end
+end
+
+function fs_lua.current_user()
+   return posix.getpwuid(posix.geteuid()).pw_name
+end
+
+--- Create a temporary directory.
+-- @param name string: name pattern to use for avoiding conflicts
+-- when creating temporary directory.
+-- @return string or (nil, string): name of temporary directory or (nil, error message) on failure.
+function fs_lua.make_temp_dir(name)
+   assert(type(name) == "string")
+   name = dir.normalize(name)
+
+   return posix.mkdtemp((os.getenv("TMPDIR") or "/tmp") .. "/luarocks_" .. name:gsub(dir.separator, "_") .. "-XXXXXX")
 end
 
 end
@@ -799,14 +807,16 @@ end
 --- Move a file.
 -- @param src string: Pathname of source
 -- @param dest string: Pathname of destination
+-- @param perms string or nil: Permissions for destination file,
+-- or nil to use the source filename permissions.
 -- @return boolean or (boolean, string): true on success, false on failure,
 -- plus an error message.
-function fs_lua.move(src, dest)
+function fs_lua.move(src, dest, perms)
    assert(src and dest)
    if fs.exists(dest) and not fs.is_dir(dest) then
       return false, "File already exists: "..dest
    end
-   local ok, err = fs.copy(src, dest)
+   local ok, err = fs.copy(src, dest, perms)
    if not ok then
       return false, err
    end
@@ -823,28 +833,29 @@ end
 -- @return boolean or (boolean, string): true on success, false on failure,
 -- plus an error message.
 function fs_lua.check_command_permissions(flags)
-   local root_dir = path.root_dir(cfg.rocks_dir)
    local ok = true
    local err = ""
-   for _, dir in ipairs { cfg.rocks_dir, root_dir } do
-      if fs.exists(dir) and not fs.is_writable(dir) then
-         ok = false
-         err = "Your user does not have write permissions in " .. dir
-         break
-      end
-   end
-   if ok and not fs.exists(root_dir) then
-      local root = fs.root_of(root_dir)
-      local parent = root_dir
-      repeat
-         parent = dir.dir_name(parent)
-         if parent == "" then
-            parent = root
+   for _, directory in ipairs { cfg.rocks_dir, cfg.deploy_lua_dir, cfg.deploy_bin_dir, cfg.deploy_lua_dir } do
+      if fs.exists(directory) then
+         if not fs.is_writable(directory) then
+            ok = false
+            err = "Your user does not have write permissions in " .. directory
+            break
+         end
+      else
+         local root = fs.root_of(directory)
+         local parent = directory
+         repeat
+            parent = dir.dir_name(parent)
+            if parent == "" then
+               parent = root
+            end
+         until parent == root or fs.exists(parent)
+         if not fs.is_writable(parent) then
+            ok = false
+            err = directory.." does not exist and your user does not have write permissions in " .. parent
+            break
          end
-      until parent == root or fs.exists(parent)
-      if not fs.is_writable(parent) then
-         ok = false
-         err = root_dir.." does not exist and your user does not have write permissions in " .. parent
       end
    end
    if ok then
